﻿using MetroLog.MicrosoftExtensions;
using MetroLog.Operators;
using StudiApp.Models;
using StudiApp.Repository;
using StudiApp.ViewModels;
using StudiApp.Views;
using Syncfusion.Maui.Core.Hosting;

namespace StudiApp;

public static class MauiProgram
{
	public static MauiApp CreateMauiApp()
	{
		var builder = MauiApp.CreateBuilder();
        builder
            .UseMauiApp<App>()
            .ConfigureSyncfusionCore()
            .ConfigureFonts(fonts =>
            {
                fonts.AddFont("OpenSans-Regular.ttf", "OpenSansRegular");
                fonts.AddFont("OpenSans-Semibold.ttf", "OpenSansSemibold");
            })
            .RegisterAppServices()
            .RegisterViewModels()
            .RegisterViews()
            .RegisterDatabase()
            .RegisterLogger();


        return builder.Build();
	}

    private static void RegisterLogger(this MauiAppBuilder mauiAppBuilder)
    {
        mauiAppBuilder.Logging.AddTraceLogger(_ => { });

        mauiAppBuilder.Logging.AddStreamingFileLogger(options =>
        {
            const string loggingFolder = "Logs";
            var path = Path.Combine(FileSystem.AppDataDirectory, loggingFolder);

            options.RetainDays = 60;
            options.FolderPath = path;
        });

        mauiAppBuilder.Services.AddSingleton(LogOperatorRetriever.Instance);
    }

    public static MauiAppBuilder RegisterAppServices(this MauiAppBuilder mauiAppBuilder)
    {
        return mauiAppBuilder;
    }

    public static MauiAppBuilder RegisterViewModels(this MauiAppBuilder mauiAppBuilder)
    {
        mauiAppBuilder.Services.AddSingleton<DashboardViewModel>();
        mauiAppBuilder.Services.AddSingleton<MarksOverviewViewModel>();
        mauiAppBuilder.Services.AddSingleton<CoursesOverviewViewModel>();
        mauiAppBuilder.Services.AddSingleton<TimetableViewModel>();
        mauiAppBuilder.Services.AddSingleton<TodoOverviewViewModel>();
        mauiAppBuilder.Services.AddSingleton<SettingsViewModel>();
        mauiAppBuilder.Services.AddSingleton<IRepository<Mark, int>, SqliteRepository<Mark>>();

        mauiAppBuilder.Services.AddTransient<CoursesEditViewModel>();
        mauiAppBuilder.Services.AddTransient<TodoEditViewModel>();
        mauiAppBuilder.Services.AddTransient<MarksEditViewModel>();

        return mauiAppBuilder;
    }
    public static MauiAppBuilder RegisterViews(this MauiAppBuilder mauiAppBuilder)
    {
        mauiAppBuilder.Services.AddSingleton<DashboardView>();
        mauiAppBuilder.Services.AddSingleton<MarksOverviewView>();
        mauiAppBuilder.Services.AddSingleton<TimetableView>();
        mauiAppBuilder.Services.AddSingleton<CoursesOverviewView>();
        mauiAppBuilder.Services.AddSingleton<TodoOverviewView>();
        mauiAppBuilder.Services.AddSingleton<SettingsView>();

        mauiAppBuilder.Services.AddTransient<CoursesEditView>();
        mauiAppBuilder.Services.AddTransient<TodoEditView>();
        mauiAppBuilder.Services.AddTransient<MarksEditView>();

        return mauiAppBuilder;
    }

    public static MauiAppBuilder RegisterDatabase(this MauiAppBuilder mauiAppBuilder)
    {
        mauiAppBuilder.Services.AddSingleton<IRepository<Mark, int>, SqliteRepository<Mark>>();
        mauiAppBuilder.Services.AddSingleton<IRepository<Lecture, int>, SqliteRepository<Lecture>>();
        mauiAppBuilder.Services.AddSingleton<IRepository<Course, int>, SqliteRepository<Course>>();
        mauiAppBuilder.Services.AddSingleton<IRepository<ToDo, int>, SqliteRepository<ToDo>>();

        return mauiAppBuilder;
    }
}
