﻿using System.Collections.ObjectModel;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using Microsoft.Extensions.Logging;
using StudiApp.Views;

namespace StudiApp.ViewModels
{
    public partial class DashboardViewModel : ObservableObject
    {
        [ObservableProperty] 
        private ObservableCollection<string> _items;

        [ObservableProperty] 
        private int _count = 0;

        [ObservableProperty]
        private string _text = "Click me!";

        private readonly ILogger _logger;

        public DashboardViewModel(ILogger<DashboardViewModel> logger)
        {
            _logger = logger;
            _items = new ObservableCollection<string>();
        }


        [RelayCommandAttribute]
        public void OnCounterClicked()
        {
            Count++;

            Text = Count == 1 ? $"Clicked {Count} time" : $"Clicked {Count} times";

            Items.Add(Text);

            _logger.LogInformation($"Clicked Counter {Count} times.");

            SemanticScreenReader.Announce(Text);
        }

        [RelayCommandAttribute]
        public void Delete(string s)
        {
            if (Items.Contains(s))
            {
                Items.Remove(s);
            }
        }

        [RelayCommandAttribute]
        async Task Tap(string s)
        {
            //No parameters
            //await Shell.Current.GoToAsync(nameof(CoursesEditView));

            //Query Expression for simple DataTypes
            //await Shell.Current.GoToAsync($"{nameof(CoursesEditView)}?Text={s}");

            //Dictionary Parameter for more complex Datatypes
            await Shell.Current.GoToAsync(nameof(CoursesEditView),
                new Dictionary<string, object>
                {
                    {"Text", s}
                });

        }

    }
}
